@@ -34,7 +34,6 @@
 #include <sys/mman.h>
 #include <unistd.h>
 
-#include "arch_common.h"
 #include "config.h"
 #include "error.h"
 #include "insn_queue_lib.h"
@@ -48,8 +47,34 @@ char __ulp_metadata_buffer[ULP_METADATA_BUF_LEN] = { 0 };
 struct ulp_metadata *__ulp_metadata_ref = NULL;
 struct ulp_detour_root *__ulp_root = NULL;
 
-/* current libpulp version.  */
-const char __ulp_version[] = VERSION;
+/* clang-format off */
+
+/** Intel endbr64 instruction optcode.  */
+static const uint8_t insn_endbr64[] = {INSN_ENDBR64};
+
+/** Offset of the data entry in the ulp_prologue.  */
+#define ULP_DATA_OFFSET 6           // ------------------------------+
+                                    //                               |
+char ulp_prologue[ULP_NOPS_LEN] = { //                               |
+  // Preceding nops                                                  |
+  0xff, 0x25, 0, 0, 0, 0,           // jmp     0x0(%rip) <-------+   |
+  0, 0, 0, 0, 0, 0, 0, 0,           // <data>  &__ulp_prolog     | <-+
+  // Function entry is here                                      |
+  0xeb, -(PRE_NOPS_LEN + 2)         // jmp ----------------------+
+  // (+2 because the previous jump consumes 2 bytes.
+};
+
+/** Offset of the data entry in the ulp_prologue.  */
+char ulp_prologue_endbr64[ULP_NOPS_LEN_ENDBR64] = {
+  // Preceding nops
+  0xff, 0x25, 0, 0, 0, 0,           // jmp     0x0(%rip) <-------+
+  0, 0, 0, 0, 0, 0, 0, 0,           // <data>  &__ulp_prolog     |
+  // Function entry is here                                      |
+  INSN_ENDBR64,                     // endbr64                   |
+  0xeb, -(PRE_NOPS_LEN + 2 + 4)     // jmp ----------------------+
+  // (+2 because the previous jump consumes 2 bytes.
+};
+/* clang-format on */
 
 unsigned int __ulp_root_index_counter = 0;
 unsigned long __ulp_global_universe = 0;
@@ -61,29 +86,6 @@ begin(void)
   msgq_push("libpulp loaded...\n");
 }
 
-/** @brief Write into memory bypassing memory protections
- *
- * The process may be launched with mprotect through seccomp, which
- * will block certain addresses to be written.  This function
- * circunvent this by queuing up an instruction to the ulp insn queue
- * for the `ulp` tool to process later.
- *
- * @param dest    Destination address
- * @param src     Source address
- * @param n       number of bytes
- * @return dest on success, NULL if error.
- */
-void *
-memwrite(void *dest, const void *src, size_t n)
-{
-  error_t e = insnq_insert_write(dest, n, src);
-  if (e != ENONE) {
-    return NULL;
-  }
-
-  return dest;
-}
-
 /** @brief Revert all live patches associated with library `lib_name`
  *
  * The user may have applied a series of live patches on a library named
@@ -418,7 +420,7 @@ load_so(char *obj)
 {
   void *patch_obj;
 
-  patch_obj = dlopen(obj, RTLD_NOW);
+  patch_obj = dlopen(obj, RTLD_LAZY);
   if (!patch_obj) {
     WARN("Unable to load shared object %s: %s.", obj, dlerror());
     return NULL;
@@ -454,7 +456,7 @@ parse_metadata(struct ulp_metadata *ulp)
     goto metadata_clean;
 
   if (!load_so_handlers(ulp)) {
-    ret = EDLOPEN;
+    ret = errno;
     goto metadata_clean;
   }
 
@@ -725,25 +727,18 @@ ulp_apply_all_units(struct ulp_metadata *ulp)
          symbol's address by its name.  */
       patch_address =
           (uintptr_t)load_so_symbol(ref->reference_name, ulp->so_handler);
-
-      if (patch_address == 0) {
-        return ENONEWFUNC;
-      }
-
       ref->patch_offset = patch_address - patch_base;
     }
     else {
       patch_address = patch_base + ref->patch_offset;
     }
-
     if (ref->tls) {
-      tls_index ti = { .ti_module = tls_idx,
-                       .ti_offset = ref->target_offset - TLS_DTV_OFFSET };
-      memwrite((void *)patch_address, &ti, sizeof(ti));
+      tls_index ti = { .ti_module = tls_idx, .ti_offset = ref->target_offset };
+      memcpy((void *)patch_address, &ti, sizeof(ti));
     }
     else {
       uintptr_t target_address = target_base + ref->target_offset;
-      memwrite((void *)patch_address, &target_address, sizeof(void *));
+      memcpy((void *)patch_address, &target_address, sizeof(void *));
     }
     ref = ref->next;
   }
@@ -833,11 +828,6 @@ ulp_state_update(struct ulp_metadata *ulp)
     unit = unit->next;
   }
 
-  /* Insert timestamp.  */
-  struct timespec t;
-  clock_gettime(CLOCK_REALTIME, &t);
-  a_patch->timestamp = t.tv_sec;
-
   /* leave last on top of list to optmize revert */
   prev_patch = __ulp_state.patches;
   __ulp_state.patches = a_patch;
@@ -958,23 +948,6 @@ check_patch_dependencies(struct ulp_metadata *ulp)
   return 0;
 }
 
-/** build_id_note structure, holding information about the build id.  */
-struct build_id_note
-{
-  /* The NHdr ELF.  See elf.h for more information.   */
-
-  /** Size of the name structure.  */
-  uint32_t namesz;
-  /** Length of the build id.  */
-  uint32_t descsz;
-  /** Unknown.  */
-  uint32_t type;
-  /** Name.  */
-  char name[4]; /* Note name for build-id is "GNU\0" */
-  /** Build id.  */
-  unsigned char build_id[0];
-};
-
 /** @brief Function used by dl_iterate_phdr to check if there are some library
  *        that matches the buildid in the `data` ulp_metadata object.
  */
@@ -982,44 +955,69 @@ int
 compare_build_ids(struct dl_phdr_info *info,
                   size_t __attribute__((unused)) size, void *data)
 {
+  int i;
+  char *note_ptr, *build_id_ptr, *note_sec;
+  uint32_t note_type, build_id_len, name_len, sec_size, next = 0;
+  struct ulp_metadata *ulp;
+  ulp = (struct ulp_metadata *)data;
 
-/** Align `val` to `align` bytes.  */
-#define ALIGN(val, align) (((val) + (align) - 1) & ~((align) - 1))
+  /* algorithm goes as follows:
+   * 1 - check every object inside ulp_metadata
+   * 1.1 - if object is main, match loaded object whose name length is 0
+   * 1.2 - else, match ulp object and loaded object names
+   * 2 - check every phdr for loaded object and match all PT_NOTE
+   * 3 - trespass PT_NOTE searching for NT_GNU_BUILD_ID
+   * 3.1 - once found, match contents with ulp object build id
+   * 3.2 - if match, mark ulp object as checked and break dl_iterate (return 1)
+   * 3.3 - else, continue looking for the library by returning 0
+   *
+   * Algorithm assumes that objects will only have one NT_GNU_BUILD_ID entry
+   */
 
-  struct ulp_metadata *ulp = (struct ulp_metadata *)data;
-
-  for (unsigned i = 0; i < info->dlpi_phnum; ++i) {
+  for (i = 0; i < info->dlpi_phnum; i++) {
     if (info->dlpi_phdr[i].p_type != PT_NOTE)
       continue;
 
-    struct build_id_note *note =
-        (struct build_id_note *)(info->dlpi_addr + info->dlpi_phdr[i].p_vaddr);
-    ptrdiff_t size = info->dlpi_phdr[i].p_filesz;
+    note_sec = (char *)(info->dlpi_phdr[i].p_vaddr + info->dlpi_addr);
+    sec_size = info->dlpi_phdr[i].p_memsz;
 
-    while (size >= (ptrdiff_t)sizeof(struct build_id_note)) {
-      if (note->type == NT_GNU_BUILD_ID && note->namesz == 4 &&
-          note->descsz >= 16) {
+    for (note_ptr = note_sec, note_type = (uint32_t) * (note_ptr + 8);
+         note_type != NT_GNU_BUILD_ID && note_ptr < note_sec + sec_size;
+         note_ptr = note_sec + next, note_type = (uint32_t) * (note_ptr + 8)) {
 
-        /* Check if build matches.  */
-        if (note->descsz == ulp->objs->build_id_len &&
-            memcmp(ulp->objs->build_id, note->build_id, note->descsz)) {
-          ulp->objs->build_id_check = 1;
-          return 1;
-        }
+      build_id_len = (uint32_t) * (note_ptr + 4);
+      name_len = (uint32_t)*note_ptr;
 
-        /* Does not match.  Go to another lib.  */
-        break;
-      }
-      size_t offset = (sizeof(uint32_t) * 3 + ALIGN(note->namesz, 4) +
-                       ALIGN(note->descsz, 4));
-      note = (struct build_id_note *)((char *)note + offset);
-      size -= offset;
+      /* fix paddings */
+      build_id_len += build_id_len % 4;
+      name_len += name_len % 4;
+
+      next = next + build_id_len + name_len + 12;
+    }
+
+    /* could not fid the build id in the note section, go to next sec */
+    if (note_type != NT_GNU_BUILD_ID)
+      continue;
+
+    build_id_len = (uint32_t) * (note_ptr + 4);
+    build_id_len += build_id_len % 4;
+    if (build_id_len != ulp->objs->build_id_len)
+      return 0;
+
+    /* we compute, but currently do not check note names */
+    name_len = (uint32_t)*note_ptr;
+    name_len += name_len % 4;
+
+    build_id_ptr = note_ptr + 12 + name_len;
+    if (memcmp(ulp->objs->build_id, build_id_ptr, build_id_len) == 0) {
+      ulp->objs->build_id_check = 1;
+      return 1;
+    }
+    else {
+      return 0;
     }
   }
-
   return 0;
-
-#undef ALIGN
 }
 
 /** @brief Check if the build id in the patch matches some .
@@ -1042,6 +1040,45 @@ check_build_id(struct ulp_metadata *ulp)
   return 1;
 }
 
+/** @brief Copy the ulp proglogue layout into the function to be patched's
+ * prologue
+ *
+ * This function copies the new code prologue into the old function prologue
+ * in order to redirect the execution to the new function.
+ *
+ */
+static void
+ulp_patch_prologue_layout(void *old_fentry, const char *prologue, int len)
+{
+  insnq_insert_write(old_fentry, len, prologue);
+}
+
+/** @brief skip the ulp prologue.
+ *
+ * When a function gets live patch, the nops at its entry point get replaced
+ * with a backwards-jump to a small segment of code that redirects execution to
+ * the new version of the function. However, when all live patches to said
+ * function are deactivated (because the live patches have been reversed), the
+ * need for the backwards-jump is gone.
+ *
+ * The following function replaces the backwards-jump with nops, thus making
+ * the target function look like it did at the beginning of execution, i.e.
+ * without live patches.
+ *
+ * @param fentry        Address to write the prologue to.
+ */
+void
+ulp_skip_prologue(void *fentry)
+{
+  static const char insn_nop2[] = { 0x66, 0x90 };
+  int bias = 0;
+  if (memcmp(fentry, insn_endbr64, sizeof(insn_endbr64)) == 0)
+    bias += sizeof(insn_endbr64);
+
+  /* Do not jump backwards on function entry (0x6690 is a nop on x86). */
+  insnq_insert_write((char *)fentry + bias, sizeof(insn_nop2), insn_nop2);
+}
+
 /** @brief Get patched address of function with universe index = idx.
  *
  * This function will get the function address (plus 2) of the function whose
@@ -1073,6 +1110,15 @@ __ulp_manage_universes(unsigned long idx)
   }
   if (!target)
     target = root->patched_addr + 2;
+
+  /* clang-format off */
+  asm (
+    "movq %0, %%r11;"
+    :
+    : "r" (target)
+    :
+  );
+  /* clang-format on */
 }
 
 /** @brief Get next root index and update the global counter.
@@ -1105,7 +1151,7 @@ push_new_detour(unsigned long universe, unsigned char *patch_id,
 
   detour = calloc(1, sizeof(struct ulp_detour));
   if (!detour) {
-    WARN("Unable to allocate memory for ulp detour");
+    WARN("Unable to acllocate memory for ulp detour");
     return 0;
   }
 
@@ -1120,6 +1166,82 @@ push_new_detour(unsigned long universe, unsigned char *patch_id,
   return 1;
 }
 
+/** @brief Write new function address into data prologue of  `old_fentry`.
+ *
+ *  This function replaces the `<data>` section in prologue `old_fentry`
+ *  with a pointer to the new function given by `manager`, which will
+ *  replace the to be patched function.
+ *
+ *  @param old_fentry Pointer to prologue of to be replaced function
+ *  @param manager Address of new function.
+ */
+void
+ulp_patch_addr_absolute(void *old_fentry, void *manager)
+{
+  char *dst = (char *)old_fentry + ULP_DATA_OFFSET;
+  insnq_insert_write(dst, sizeof(void *), &manager);
+}
+
+/** @brief Actually patch the old function with the new function
+ *
+ * This function will finally patch the old function pointed by `old_faddr`
+ * with the one pointed by `new_faddr`, replacing the ulp NOP prologue with
+ * the intended content to redirect to the new function.
+ *
+ * @param old_faddr     Address of the old function.
+ * @param new_faddr     Address of the new function.
+ * @param enable        False to disable the redirection to the new function.
+ *
+ * @return              0 if success, error code otherwise.
+ */
+int
+ulp_patch_addr(void *old_faddr, void *new_faddr, int enable)
+{
+  void *addr;
+
+  int ulp_nops_len;
+  const char *prologue;
+
+  const unsigned char *as_bytes = old_faddr;
+
+  /* Check if the first instruction of old_function is endbr64.  In this
+     case, we have to handle things differently.  */
+  if (memcmp(old_faddr, insn_endbr64, sizeof(insn_endbr64)) == 0) {
+    ulp_nops_len = ULP_NOPS_LEN_ENDBR64;
+    prologue = ulp_prologue_endbr64;
+    as_bytes += sizeof(insn_endbr64);
+  }
+  else {
+    ulp_nops_len = ULP_NOPS_LEN;
+    prologue = ulp_prologue;
+  }
+
+  /* Check if we have the two NOP sequence or a JMP ref8 insn.  Else we might
+     be attempting to patch a non-livepatchable function.  */
+
+  if (!(as_bytes[0] == 0xEB ||
+        (as_bytes[1] == 0x90 &&
+         (as_bytes[0] == 0x90 || as_bytes[0] == 0x66)))) {
+    WARN("Function at addr %lx is not livepatchable",
+         (unsigned long)old_faddr);
+    return ENOPATCHABLE;
+  }
+
+  /* Find the starting address of the pages containing the nops. */
+  addr = old_faddr - PRE_NOPS_LEN;
+
+  /* Actually patch the prologue. */
+  if (enable) {
+    ulp_patch_prologue_layout(addr, prologue, ulp_nops_len);
+    ulp_patch_addr_absolute(addr, new_faddr);
+  }
+  else {
+    ulp_skip_prologue(old_faddr);
+  }
+
+  return 0;
+}
+
 struct ulp_applied_patch *
 ulp_get_applied_patch(const unsigned char *id)
 {
