@@ -42,28 +42,5 @@ child.expect('8-7-6-5-4-3-2-1', reject='1-2-3-4-5-6-7-8');
 child.expect('10.0-9.0-8.0-7.0-6.0-5.0-4.0-3.0-2.0-1.0',
              reject='1.0-2.0-3.0-4.0-5.0-6.0-7.0-8.0-9.0-10.0');
 
-child.livepatch('.libs/libparameters_livepatch1.so', revert=True)
-
-child.sendline('')
-child.expect('1-2-3-4-5-6-7-8');
-child.expect('1.0-2.0-3.0-4.0-5.0-6.0-7.0-8.0-9.0-10.0',
-             reject='10.0-9.0-8.0-7.0-6.0-5.0-4.0-3.0-2.0-1.0');
-
 child.close(force=True)
-
-# Now test things related to references and visibility.
-child = testsuite.spawn('block_mprotect visibility', script=False)
-
-child.expect('Press ENTER to continue')
-child.sendline('')
-child.expect('This is a hidden string')
-child.expect('Press ENTER to continue')
-
-child.livepatch('.libs/libvisibility_livepatch2.so')
-
-child.sendline('')
-child.expect('String from lp This is a hidden string')
-
-child.close(force=True)
-
 exit(0)
