@@ -86,17 +86,6 @@ void
 ulp_warn(const char *format, ...)
 {
   if (!ulp_quiet) {
-    time_t rawtime;
-    struct tm *timeinfo, local_tm;
-    char timestamp[32];
-
-    time(&rawtime);
-    timeinfo = localtime_r(&rawtime, &local_tm);
-
-    strftime(timestamp, 32, "[%Y-%m-%d %H:%M:%S] ", timeinfo);
-
-    fputs(timestamp, stderr);
-
     va_list args;
     va_start(args, format);
     vfprintf(stderr, format, args);
@@ -108,17 +97,6 @@ void
 ulp_debug(const char *format, ...)
 {
   if (ulp_verbose) {
-    time_t rawtime;
-    struct tm *timeinfo, local_tm;
-    char timestamp[32];
-
-    time(&rawtime);
-    timeinfo = localtime_r(&rawtime, &local_tm);
-
-    strftime(timestamp, 32, "[%Y-%m-%d %H:%M:%S] ", timeinfo);
-
-    fputs(timestamp, stderr);
-
     va_list args;
     va_start(args, format);
     vfprintf(stderr, format, args);
@@ -211,8 +189,12 @@ release_ulp_dynobj(struct ulp_dynobj *obj)
 
   for (; obj != NULL; obj = nexto) {
     nexto = obj->next;
-    FREE_AND_NULLIFY(obj->filename);
-    FREE_AND_NULLIFY(obj->thread_states);
+
+    if (obj->filename) {
+      free(obj->filename);
+    }
+    if (obj->thread_states)
+      free(obj->thread_states);
     free(obj);
   }
 }
@@ -566,7 +548,7 @@ parse_dynobj_elf_headers(int pid, struct ulp_dynobj *obj)
   if (ehdr_addr == 0) {
     /* If l_addr is zero, it means that there is no load bias.  In that case,
      * the elf address is on address 0x400000 on x86_64.  */
-    ret = read_memory((char *)&ehdr, sizeof(ehdr), pid, EXECUTABLE_START);
+    ret = read_memory((char *)&ehdr, sizeof(ehdr), pid, 0x400000UL);
   }
   else {
     ret = read_memory((char *)&ehdr, sizeof(ehdr), pid, ehdr_addr);
@@ -585,7 +567,7 @@ parse_dynobj_elf_headers(int pid, struct ulp_dynobj *obj)
   /* Get first process header address.  */
   phdr_addr = ehdr_addr + ehdr.e_phoff;
   if (ehdr_addr == 0)
-    phdr_addr += EXECUTABLE_START;
+    phdr_addr += 0x400000UL;
 
   /* Iterate over each process header.  */
   for (i = 0; i < ehdr.e_phnum; i++) {
@@ -866,21 +848,16 @@ get_libpulp_extern_symbols(struct ulp_dynobj *obj, int pid)
         obj->enable_disable_patching = ehdr_addr + sym.st_value;
         bitfield |= (1 << 8);
       }
+
       else if (!strcmp(remote_name, "_insn_queue")) {
         obj->insn_queue = ehdr_addr + sym.st_value;
         bitfield |= (1 << 9);
       }
-      else if (!strcmp(remote_name, "_version")) {
-        char str[32];
-        read_string_allocated(str, sizeof(str), pid, ehdr_addr + sym.st_value);
-        obj->libpulp_version = ulp_version_from_string(str);
-        bitfield |= (1 << 10);
-      }
     }
 
     dynsym_addr += sizeof(sym);
 
-    if (bitfield == 0x7FF)
+    if (bitfield == 0x3FF)
       break;
   }
 
@@ -1081,7 +1058,7 @@ parse_main_dynobj(struct ulp_process *process)
   }
 
   /* calloc initializes all to zero */
-  obj = calloc(1, sizeof(struct ulp_dynobj));
+  obj = calloc(sizeof(struct ulp_dynobj), 1);
   if (!obj) {
     DEBUG("unable to allocate memory.");
     return ENOMEM;
@@ -1151,8 +1128,7 @@ parse_lib_dynobj(struct ulp_dynobj *obj, struct ulp_process *process)
     return 0;
 
   /* Pointers to linux-vdso.so are invalid, so skip this library.  */
-  if (strcmp(obj->filename, "linux-vdso.so.1") &&
-      strcmp(obj->filename, "linux-vdso64.so.1"))
+  if (strcmp(obj->filename, "linux-vdso.so.1"))
     parse_dynobj_elf_headers(pid, obj);
 
   /* Only libpulp.so should have those symbols exported.  */
@@ -1182,7 +1158,7 @@ get_libname_dynobj(struct ulp_process *process, struct link_map *link_map_addr)
   int pid = process->pid;
 
   /* calloc initializes all to zero */
-  obj = calloc(1, sizeof(struct ulp_dynobj));
+  obj = calloc(sizeof(struct ulp_dynobj), 1);
 
   if (read_memory((char *)&obj->link_map, sizeof(struct link_map), pid,
                   (Elf64_Addr)link_map_addr)) {
@@ -1273,22 +1249,29 @@ initialize_data_structures(struct ulp_process *process)
   ulp_error_t ret = 0;
 
   if (!process)
-    return EINVAL;
+    return 1;
 
   DEBUG("getting in-memory information about process %d.", process->pid);
 
   if (attach(process->pid)) {
-    ret = ETARGETHOOK;
+    DEBUG("Unable to attach to %d to read data.\n", process->pid);
+    ret = 1;
     goto detach_process;
   }
 
   ret = parse_main_dynobj(process);
   if (ret) {
+    WARN("unable to get in-memory information about the main executable: %s\n",
+         libpulp_strerror(ret));
+    ret = 1;
     goto detach_process;
   }
 
   ret = parse_libs_dynobj(process);
   if (ret) {
+    WARN("unable to get in-memory information about shared libraries: %s\n",
+         libpulp_strerror(ret));
+    ret = 1;
     goto detach_process;
   }
 
@@ -1297,7 +1280,7 @@ initialize_data_structures(struct ulp_process *process)
   if (read_memory((char *)&ulp_state, sizeof(ulp_state), process->pid,
                   process->dynobj_libpulp->state) ||
       ulp_state.load_state == 0) {
-    DEBUG("libpulp not ready (constructors not yet run). Try again later.");
+    WARN("libpulp not ready (constructors not yet run). Try again later.");
     ret = EAGAIN;
     goto detach_process;
   }
@@ -1468,7 +1451,7 @@ hijack_threads(struct ulp_process *process)
          *   Save all registers;
          *   Update the list.
          */
-        t = calloc(1, sizeof(struct ulp_thread));
+        t = calloc(sizeof(struct ulp_thread), 1);
         if (!t) {
           WARN("unable to allocate memory.");
           goto child_alloc_error;
@@ -1556,7 +1539,7 @@ patch_applied(struct ulp_process *process, unsigned char *id, int *result)
 {
   int ret;
   struct ulp_thread *thread;
-  registers_t context;
+  struct user_regs_struct context;
   ElfW(Addr) routine;
 
   DEBUG("checking if live patch is already applied.");
@@ -1584,7 +1567,7 @@ patch_applied(struct ulp_process *process, unsigned char *id, int *result)
   if (ret)
     return ret;
 
-  *result = FUNCTION_RETURN_REG(context);
+  *result = context.rax;
   return 0;
 }
 
@@ -1604,7 +1587,7 @@ apply_patch(struct ulp_process *process, void *metadata, size_t metadata_size)
 {
   int ret;
   struct ulp_thread *thread;
-  registers_t context;
+  struct user_regs_struct context;
   ElfW(Addr) routine;
 
   struct ulp_dynobj *dynobj_libpulp = process->dynobj_libpulp;
@@ -1626,22 +1609,22 @@ apply_patch(struct ulp_process *process, void *metadata, size_t metadata_size)
   context = thread->context;
   routine = dynobj_libpulp->trigger;
 
-  rax = FUNCTION_RETURN_REG(context);
+  rax = context.rax;
 
   DEBUG(">>> running libpulp functions within target process...");
   ret = run_and_redirect(thread->tid, &context, routine);
   if (ret) {
     WARN("error during live patch application: %s",
-         libpulp_strerror(FUNCTION_RETURN_REG(context)));
+         libpulp_strerror(context.rax));
   }
   DEBUG(">>> done.");
   if (ret)
     return ret;
 
-  if (FUNCTION_RETURN_REG(context) != 0) {
-    if (FUNCTION_RETURN_REG(context) == EAGAIN)
+  if (context.rax != 0) {
+    if (context.rax == EAGAIN)
       DEBUG("patching failed in libpulp.so: libc/libdl locks were busy");
-    else if (FUNCTION_RETURN_REG(context) == rax) {
+    else if (context.rax == rax) {
       /* If rax register is not changed in this process, it is evidence that
          the routine in libpulp.so wasn't executed by some reason.  */
       DEBUG("patching failed in libpulp.so: %s",
@@ -1650,7 +1633,7 @@ apply_patch(struct ulp_process *process, void *metadata, size_t metadata_size)
     }
     else
       DEBUG("patching failed in libpulp.so: %s",
-            libpulp_strerror(FUNCTION_RETURN_REG(context)));
+            libpulp_strerror(context.rax));
   }
 
   /* Interpret now any code that libpulp send to us.  */
@@ -1660,7 +1643,7 @@ apply_patch(struct ulp_process *process, void *metadata, size_t metadata_size)
     return ret;
   }
 
-  return FUNCTION_RETURN_REG(context);
+  return context.rax;
 }
 
 int
@@ -1668,7 +1651,7 @@ revert_patches_from_lib(struct ulp_process *process, const char *lib_name)
 {
   int ret;
   struct ulp_thread *thread;
-  registers_t context;
+  struct user_regs_struct context;
   ElfW(Addr) routine;
 
   DEBUG("beginning patches removal.");
@@ -1704,13 +1687,13 @@ revert_patches_from_lib(struct ulp_process *process, const char *lib_name)
   if (ret)
     return ret;
 
-  if (FUNCTION_RETURN_REG(context) != 0) {
-    if (FUNCTION_RETURN_REG(context) == EAGAIN)
+  if (context.rax != 0) {
+    if (context.rax == EAGAIN)
       DEBUG("patches reverse-all failed in libpulp.so: libc/libdl locks were "
             "busy");
     else
       DEBUG("patches reverse-all failed in libpulp.so: %s",
-            libpulp_strerror(FUNCTION_RETURN_REG(context)));
+            libpulp_strerror(context.rax));
   }
 
   /* Interpret now any code that libpulp send to us.  */
@@ -1720,7 +1703,7 @@ revert_patches_from_lib(struct ulp_process *process, const char *lib_name)
     return ret;
   }
 
-  return FUNCTION_RETURN_REG(context);
+  return context.rax;
 }
 
 /* Reads the global universe counter in PROCESS. Returns the
@@ -1730,7 +1713,7 @@ int
 read_global_universe(struct ulp_process *process)
 {
   struct ulp_thread *thread;
-  registers_t context;
+  struct user_regs_struct context;
   ElfW(Addr) routine;
 
   if (!process->all_threads_hijacked) {
@@ -1747,7 +1730,7 @@ read_global_universe(struct ulp_process *process)
     return -1;
   };
 
-  process->global_universe = FUNCTION_RETURN_REG(context);
+  process->global_universe = context.rax;
   return 0;
 }
 
@@ -1847,10 +1830,8 @@ extract_ulp_from_so_to_mem(const char *livepatch, bool revert, char **out,
 
   /* Get full path to patch buffer.  */
   if (realpath(livepatch, &path_buffer[path_size]) == NULL) {
-    /* If we can't figure out the realpath, then use the path it was given to
-       us.  */
-    DEBUG("Unable to retrieve realpath to %s: %s", livepatch, libpulp_strerror(errno));
-    strcpy(&path_buffer[path_size], livepatch);
+    WARN("Unable to retrieve realpath to %s", livepatch);
+    return 0;
   }
   path_size = strlen(path_buffer) + 1;
 
@@ -1870,9 +1851,6 @@ extract_ulp_from_so_to_mem(const char *livepatch, bool revert, char **out,
   char *final_meta = (char *)malloc(meta_size);
   char *meta_head = final_meta;
 
-
-  /* FIXME: This is absurdly awkward.  */
-
   /* Copy the final metadata into final_meta buffer.  Things works here as
    * follows:
    *
@@ -2454,7 +2432,7 @@ error_path_context:
  * @return A `ulp_applied_patch` linked list.
  */
 static struct ulp_applied_patch *
-read_ulp_applied_patch(Elf64_Addr addr, pid_t pid, ulp_version_t version)
+read_ulp_applied_patch(Elf64_Addr addr, pid_t pid)
 {
   struct ulp_applied_patch *a_state;
 
@@ -2467,14 +2445,7 @@ read_ulp_applied_patch(Elf64_Addr addr, pid_t pid, ulp_version_t version)
     return NULL;
   }
 
-  size_t a_patch_size = sizeof(*a_state);
-
-  /* In pre 0.3.1 versions of libpulp, there was no timestamp.  */
-  if (version < ULP_VERSION_TRIPLET(0ULL, 3ULL, 1ULL)) {
-    a_patch_size -= sizeof(a_state->timestamp);
-  }
-
-  if (read_memory((char *)a_state, a_patch_size, pid, addr)) {
+  if (read_memory((char *)a_state, sizeof(*a_state), pid, addr)) {
     WARN("error reading patch state.");
     free(a_state);
     return NULL;
@@ -2490,14 +2461,13 @@ read_ulp_applied_patch(Elf64_Addr addr, pid_t pid, ulp_version_t version)
                 (Elf64_Addr)a_state->container_name);
   }
 
-
   /* ulp_applied_unit and ulp_applied_patch is not used, so don't read it. But
      set it to NULL to avoid dangling pointers.  */
 
   a_state->units = NULL;
   a_state->deps = NULL;
 
-  a_state->next = read_ulp_applied_patch((Elf64_Addr)a_state->next, pid, version);
+  a_state->next = read_ulp_applied_patch((Elf64_Addr)a_state->next, pid);
 
   return a_state;
 }
@@ -2557,8 +2527,7 @@ ulp_read_state(struct ulp_process *process)
     goto detach_process;
   }
 
-  ret = read_ulp_applied_patch((Elf64_Addr)state.patches, pid,
-                               process->dynobj_libpulp->libpulp_version);
+  ret = read_ulp_applied_patch((Elf64_Addr)state.patches, pid);
 
 detach_process:
   if (detach(process->pid)) {
