@@ -45,7 +45,7 @@
 bool enable_threading = false;
 static bool original_enable_threading;
 
-static int insert_target_process(int pid, struct ulp_process **list);
+void insert_target_process(int pid, struct ulp_process **list);
 
 static void *
 generate_ulp_list_thread(void *args)
@@ -131,10 +131,6 @@ process_list_next(struct ulp_process_iterator *it)
     symbols.  */
 static pthread_t process_list_thread;
 
-/** In case of `ulp patches -p <pid>`, we want to return an error for the
-    caller if something went wrong.  */
-static error_t any_error = ENONE;
-
 struct ulp_process *
 process_list_begin(struct ulp_process_iterator *it,
                    const char *procname_wildcard, const char *user_wildcard)
@@ -151,7 +147,7 @@ process_list_begin(struct ulp_process_iterator *it,
   if (isnumber(procname_wildcard)) {
     /* If wildcard is actually a number, then treat it as a PID.  */
     pid = atoi(procname_wildcard);
-    any_error = insert_target_process(pid, &it->last);
+    insert_target_process(pid, &it->last);
     it->now = it->last;
 
     /* Disable threading in this case.  */
@@ -184,11 +180,10 @@ process_list_end(struct ulp_process_iterator *it)
 {
   if (it->now == NULL) {
     release_ulp_process(it->last);
+    producer_consumer_delete(it->pcqueue);
     if (enable_threading) {
-      /* Make sure the threads stopped before destroying the queue.  */
       pthread_join(process_list_thread, NULL);
     }
-    producer_consumer_delete(it->pcqueue);
 
     /* In case threads were disabled because of some special case, then enable
        it now.  */
@@ -383,17 +378,42 @@ print_lib_patches(struct ulp_applied_patch *patch, const char *libname)
 
   while (patch) {
     if (!strcmp(libname, patch->lib_name)) {
-      struct tm *timeinfo = localtime(&patch->timestamp);
-      printf("      livepatch: %s", get_basename(patch->container_name));
-      if (patch->timestamp != 0) {
-        printf(" loaded on %s", asctime(timeinfo));
-      }
+      printf("      livepatch: %s\n", get_basename(patch->container_name));
       print_relevant_labels(patch->container_name);
     }
     patch = patch->next;
   }
 }
 
+/** @brief Check if function at `sym_address` has the NOP preamble.
+ *
+ * Functions that are livepatchable has ULP_NOPS_LEN - PRE_NOPS_LEN at the
+ * beginning of the function. Check the existence of this preamble.
+ *
+ * @param sym_address  Address of function in target process.
+ * @param pid          Pid of the target process.
+ *
+ * @return  True if preamble exists, false if not.
+ */
+static bool
+check_preamble(ElfW(Addr) sym_address, pid_t pid)
+{
+  unsigned char bytes[2];
+
+  if (read_memory((char *)bytes, 2, pid, sym_address)) {
+    /* In case it was unable to read the symbol due to permission error, just
+     * warn in debug output.  */
+    DEBUG("Unable to read symbol preamble at address %lx in process %d",
+          sym_address, pid);
+    return false;
+  }
+
+  /* Check for NOP NOP or XGCH AX, AX.  */
+  if ((bytes[0] == 0x90 || bytes[0] == 0x66) && bytes[1] == 0x90)
+    return true;
+  return false;
+}
+
 /** @brief Check if `libname` has a livepatch loaded.
  *
  * Check if the library with name `libname` has a livepatch loaded in the
@@ -424,20 +444,6 @@ has_livepatch_loaded(struct ulp_applied_patch *patch, const char *libname)
   return false;
 }
 
-/** @brief Check if function at `sym_address` has the NOP preamble.
- *
- * Functions that are livepatchable has ULP_NOPS_LEN - PRE_NOPS_LEN at the
- * beginning of the function. Check the existence of this preamble.
- *
- * This function is arch specific.
- *
- * @param sym_address  Address of function in target process.
- * @param pid          Pid of the target process.
- *
- * @return  True if preamble exists, false if not.
- */
-bool check_preamble(ElfW(Addr) sym_address, pid_t pid);
-
 /** @brief Check if library in `obj` on target process is livepatchable.
  *
  * Check on the target process with `pid` if the library on `obj` is
@@ -475,7 +481,7 @@ is_library_livepatchable(struct ulp_applied_patch *patch,
   if (ehdr_addr == 0) {
     /* If l_addr is zero, it means that there is no load bias.  In that case,
      * the elf address is on address 0x400000 on x86_64.  */
-    ehdr_addr = EXECUTABLE_START;
+    ehdr_addr = 0x400000UL;
   }
 
   /* FIXME: Some applications take a very long time to decide if library is
@@ -541,19 +547,12 @@ print_remote_err_status(struct ulp_process *p)
   change_color(TERM_COLOR_RESET);
 }
 
-static void
-print_process(struct ulp_process *process, int print_buildid,
-              bool only_livepatched)
+void
+print_process(struct ulp_process *process, int print_buildid)
 {
   struct ulp_dynobj *object_item;
   pid_t pid = process->pid;
   struct ulp_applied_patch *patch = ulp_read_state(process);
-
-  /* In case the flag `only_patched` is NULL, then skip process.  */
-  if (only_livepatched && patch == NULL) {
-    return;
-  }
-
   printf("PID: %d, name: %s\n", pid, get_process_name(process));
   print_remote_err_status(process);
   printf("  Livepatchable libraries:\n");
@@ -565,11 +564,6 @@ print_process(struct ulp_process *process, int print_buildid,
       printf("    in %s", object_item->filename);
       if (print_buildid)
         printf(" (%s)", buildid_to_string(object_item->build_id));
-      if (object_item->libpulp_version) {
-        const char *version =
-            ulp_version_as_string(object_item->libpulp_version);
-        printf(" (version %s)", version);
-      }
       printf(":\n");
 
       print_lib_patches(patch, object_item->filename);
@@ -610,7 +604,7 @@ has_libpulp_loaded(pid_t pid)
 /* Inserts a new process structure into LIST if the process identified
  * by PID is live-patchable.
  */
-static int
+void
 insert_target_process(int pid, struct ulp_process **list)
 {
   struct ulp_process *new = NULL;
@@ -621,16 +615,14 @@ insert_target_process(int pid, struct ulp_process **list)
   new->pid = pid;
   ret = initialize_data_structures(new);
   if (ret) {
-    WARN("error gathering target process information: %s",
-         libpulp_strerror(ret));
+    WARN("error gathering target process information.");
     release_ulp_process(new);
-    return ret;
+    return;
   }
   else {
     new->next = *list;
     *list = new;
   }
-  return 0;
 }
 
 /** @brief Prints all the info collected about the processes in `process_list`.
@@ -645,7 +637,7 @@ print_process_list(struct ulp_process *process_list, int print_buildid)
 
   process_item = process_list;
   while (process_item) {
-    print_process(process_item, print_buildid, false);
+    print_process(process_item, print_buildid);
     process_item = process_item->next;
   }
 }
@@ -654,7 +646,6 @@ int
 run_patches(struct arguments *arguments)
 {
   bool print_build_id = arguments->buildid;
-  bool only_livepatched = arguments->only_livepatched;
   ulp_quiet = arguments->quiet;
   ulp_verbose = arguments->verbose;
   enable_threading = !arguments->disable_threads;
@@ -662,12 +653,11 @@ run_patches(struct arguments *arguments)
   const char *user_wildcard = arguments->user_wildcard;
 
   struct ulp_process *p;
-  any_error = ENONE;
 
   FOR_EACH_ULP_PROCESS_FROM_USER_WILDCARD(p, process_wildcard, user_wildcard)
   {
-    print_process(p, print_build_id, only_livepatched);
+    print_process(p, print_build_id);
   }
 
-  return any_error == 0 ? 0 : 1;
+  return 0;
 }
